@@ -63,6 +63,7 @@ ongoing_hot_spare_timers = dict()
 
 DEFAULT_REQ_TIMEOUT = 1
 SERVER_HEARTBEAT_TOPIC = "/heartbeat/move_it_server"
+CLIENT_HEARTBEAT_TOPIC_BASE = "/heartbeat/move_it"
 
 COMPRESSED_ENDINGS = {'xrit': ['C_'],
                       'tar': ['.tar', '.tar.gz', '.tgz', '.tar.bz2'],
@@ -180,7 +181,6 @@ class Listener(Thread):
 
                 while self.running:
                     # Loop for restart.
-
                     LOGGER.debug("Starting listener %s", str(self.address))
                     self.create_subscriber()
 
@@ -206,6 +206,7 @@ class Listener(Thread):
                         if msg.type == "beat":
                             self.death_count = 0
                             continue
+
                         # Handle public "push" messages as a hot spare client
                         if msg.type == "push":
                             # TODO: these need to be checked and acted if
@@ -213,12 +214,22 @@ class Listener(Thread):
                             # client and are not cleared
                             LOGGER.debug("Primary client published 'push'")
                             add_to_ongoing(msg)
+                            continue
 
                         # Handle public "ack" messages as a hot spare client
                         if msg.type == "ack":
                             LOGGER.debug("Primary client finished transfer")
                             _ = add_to_file_cache(msg)
                             _ = clean_ongoing_transfer(get_msg_uid(msg))
+                            continue
+
+                        # Ignore "file" messages which don't have a request address
+                        if msg.type == "file" and "request_address" not in msg.data:
+                            LOGGER.debug("Ignoring 'file' message from primary client.")
+                            add_to_ongoing(msg)
+                            _ = add_to_file_cache(msg)
+                            _ = clean_ongoing_transfer(get_msg_uid(msg))
+                            continue
 
                         # If this is a hot spare client, wait for a while
                         # for a public "push" message which will update
@@ -266,8 +277,7 @@ def unpack_tar(filename, **kwargs):
     fnames = tuple(os.path.join(destdir, member.name) for member in members)
     if len(fnames) == 1:
         return fnames[0]
-    else:
-        return fnames
+    return fnames
 
 
 def unpack_xrit(filename, **kwargs):
@@ -343,9 +353,9 @@ def resend_if_local(msg, publisher):
         urlobj = urlparse(uri)
         if not publisher or not socket.gethostbyname(urlobj.netloc) in get_local_ips():
             return
-    else:
-        LOGGER.debug('Sending: %s', str(msg))
-        publisher.send(str(msg))
+
+    LOGGER.debug('Sending: %s', str(msg))
+    publisher.send(str(msg))
 
 
 def create_push_req_message(msg, destination, login):
@@ -530,9 +540,8 @@ def add_to_ongoing(msg):
         if huid in ongoing_transfers:
             ongoing_transfers[huid].append(msg)
             return None
-        else:
-            ongoing_transfers[huid] = [msg]
-            return huid
+        ongoing_transfers[huid] = [msg]
+        return huid
 
 
 def add_to_file_cache(msg):
@@ -544,15 +553,16 @@ def add_to_file_cache(msg):
                 file_cache.append(uid)
 
 
-def request_push(msg, destination, login=None, sync_publisher=None, publisher=None, **kwargs):
+def request_push(msg_in, destination, login=None, publisher=None, **kwargs):
     """Request a push for data."""
-    huid = add_to_ongoing(msg)
+    huid = add_to_ongoing(msg_in)
     if huid is None:
         return
 
-    if already_received(msg):
+    if already_received(msg_in):
         timeout = float(kwargs["req_timeout"])
-        send_ack(msg, timeout)
+        send_ack(msg_in, timeout)
+        _ = clean_ongoing_transfer(huid)
         return
 
     for msg in iterate_messages(huid):
@@ -572,20 +582,18 @@ def request_push(msg, destination, login=None, sync_publisher=None, publisher=No
         timeout = float(kwargs["transfer_req_timeout"])
         local_dir = create_local_dir(_destination, kwargs.get('ftp_root', '/'))
 
-        if sync_publisher:
-            sync_publisher.send(str(fake_req))
+        publisher.send(str(fake_req))
         response, hostname = send_request(msg, req, timeout)
 
         if response and response.type in ['file', 'collection', 'dataset']:
             LOGGER.debug("Server done sending file")
             add_to_file_cache(msg)
-            if sync_publisher:
-                # Send an 'ack' message so that possible hot spares know
-                # the primary has completed the request
-                msg = Message(msg.subject, 'ack', msg.data)
-                LOGGER.debug("Sending a public 'ack' of completed transfer: %s",
-                             str(msg))
-                sync_publisher.send(str(msg))
+            # Send an 'ack' message so that possible hot spares know
+            # the primary has completed the request
+            msg = Message(msg.subject, 'ack', msg.data)
+            LOGGER.debug("Sending a public 'ack' of completed transfer: %s",
+                         str(msg))
+            publisher.send(str(msg))
             try:
                 lmsg = unpack_and_create_local_message(response, local_dir,
                                                        **kwargs)
@@ -620,6 +628,7 @@ class Chain(Thread):
         super(Chain, self).__init__()
         self._config = config
         self._name = name
+        self._np = None
         self.publisher = None
         self.listeners = {}
         self.listener_died_event = Event()
@@ -630,24 +639,25 @@ class Chain(Thread):
             nameservers = self._config["nameservers"]
             if nameservers:
                 nameservers = nameservers.split()
-            self.publisher = NoisyPublisher(
+            self._np = NoisyPublisher(
                 "move_it_" + self._name,
                 port=self._config["publish_port"],
                 nameservers=nameservers)
-            self.publisher.start()
+            self.publisher = self._np.start()
         except (KeyError, NameError):
             pass
 
-    def setup_listeners(self, callback, sync_publisher):
+    def setup_listeners(self, callback):
         """Set up the listeners."""
         self.callback = callback
-        self.sync_publisher = sync_publisher
         try:
             topics = []
             if "topic" in self._config:
                 topics.append(self._config["topic"])
             if self._config.get("heartbeat", False):
                 topics.append(SERVER_HEARTBEAT_TOPIC)
+                # Subscribe also to heartbeat messages of other clients
+                topics.append(CLIENT_HEARTBEAT_TOPIC_BASE + '_' + self._name)
             for provider in self._config["providers"]:
                 if '/' in provider.split(':')[-1]:
                     parts = urlparse(provider)
@@ -666,7 +676,6 @@ class Chain(Thread):
                     provider,
                     topics,
                     callback,
-                    sync_publisher=sync_publisher,
                     publisher=self.publisher,
                     die_event=self.listener_died_event,
                     **self._config)
@@ -727,20 +736,20 @@ class Chain(Thread):
     def stop(self):
         """Stop the chain."""
         self.running = False
-        if self.publisher:
-            self.publisher.stop()
+        if self._np:
+            self._np.stop()
         self.reset_listeners()
 
     def restart(self):
         """Restart the chain, return a new running instance."""
         self.stop()
         new_chain = self.__class__(self._name, self._config)
-        new_chain.setup_listeners(self.callback, self.sync_publisher)
+        new_chain.setup_listeners(self.callback)
         new_chain.start()
         return new_chain
 
 
-def reload_config(filename, chains, callback=request_push, sync_publisher=None):
+def reload_config(filename, chains, callback=request_push):
     """Rebuild chains if needed (if the configuration changed) from *filename*."""
     LOGGER.debug("New config file detected: %s", filename)
 
@@ -760,7 +769,7 @@ def reload_config(filename, chains, callback=request_push, sync_publisher=None):
             verb = "Added"
             LOGGER.debug("Adding %s", key)
         chains[key] = Chain(key, new_config)
-        chains[key].setup_listeners(callback, sync_publisher)
+        chains[key].setup_listeners(callback)
         chains[key].start()
 
         LOGGER.debug("%s %s", verb, key)
